------------------------------
-- Localized Lua globals
------------------------------
local _G = getfenv(0)

--local tostring = _G.tostring
--local tonumber = _G.tonumber

local pairs, ipairs = _G.pairs, _G.ipairs
local select = _G.select

--local table   = _G.table
--local tconcat = table.concat
local insert = table.insert

--local bit = _G.bit

local string    = _G.string
local format = string.format
--local strfind   = string.find
--local strmatch  = string.match
--local strlower  = string.lower

------------------------------
-- Localized Blizzard API
------------------------------

------------------------------
-- AddOn Namespace
------------------------------
local LibStub = _G.LibStub
local MODNAME = "Skill Guide"
local addon   = LibStub("AceAddon-3.0"):NewAddon(MODNAME, "AceConsole-3.0", "AceEvent-3.0")
_G.SkillGuide = addon

local L = LibStub("AceLocale-3.0"):GetLocale(MODNAME)

-- local DataStore = LibStub("AceAddon-3.0"):GetAddon("DataStore");

-- Get the private table
local private = select(2, ...)

-- Global Options
addon.optionsFrame = {}

------------------------------
-- Check for the required libriaries, Blizzard
-- already checked for dependencies
------------------------------
local MissingLibs
do
  local REQUIRED_LIBS = {
    "AceConfig-3.0",
    "AceDB-3.0",
    "AceDBOptions-3.0",
    "AceGUI-3.0",
    "CallbackHandler-1.0",
    "LibAboutPanel",
  }
  function MissingLibs()
    local missing = false
    
    for idx, lib in ipairs(REQUIRED_LIBS) do
      if not LibStub:GetLibrary(lib, true) then
        missing = true
        addon:Print(strformat(L["MISSING_LIB"], lib))
      end
    end
    
    return missing
  end
end -- do

if MissingLibs() then
  --[===[@alpha@
  addon:Print("You are using an alpha version of Skill Guide. Not all required libs are installed. Please install the missing libraries and restart.")
  --@end-alpha@]===]
  _G.SkillGuide = nil
  return
end

function addon:OnInitialize()
  --[===[@alpha@
  addon:Print("Initializing...")
  --@end-alpha@]===]
  
  local defaults = {
    char = {
      tradeskills = {
         ['*'] = {  -- ["spellID"]
            optimal = {},
         },
      },
    },
    profile = {
      frameopts = {},
    },
  }  
    
  self.db = LibStub("AceDB-3.0"):New("SkillGuideDB", defaults)
  if not self.db then
    self:Print("SavedVariables not loaded")
    return
  end
  
  local version = GetAddOnMetadata("SkillGuide", "Version")
  self.version = version
  
  self:SetupOptions()
  
  -- Register slash command
  self:RegisterChatCommand("skillguide", "ChatCommand")
  self:RegisterChatCommand("sg", "ChatCommand")
end

function addon:OnEnable()
  self:Print(MODNAME .. " enabled...")
  
  private.character = DataStore:GetCharacter()
  
  -- Register for events
  addon:RegisterEvent("TRADE_SKILL_SHOW")
end

function addon:OnDisable()
  self:UnregisterEvent("TRADE_SKILL_SHOW")
end

function addon:ChatCommand(input)
  -- Open the Blizzard Interface Addon Panel if there is no input
  -- or /sg faq
  if not input or (input and input:trim() == "") then
    InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
  elseif (input == strlower(L["About"])) then
    if (self.optionsFrame["About"]) then
      InterfaceOptionsFrame_OpenToCategory(self.optionsFrame["About"])
    else
      InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
    end
  elseif (input == strlower(L["Profile"])) then
    if (self.optionsFrame["Profile"]) then
      InterfaceOptionsFrame_OpenToCategory(self.optionsFrame["Profiles"])
    else
      InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
    end
  end
end

