------------------------------
-- Localize Lua Globals
------------------------------
local _G = getfenv(0)
local select = _G.select
local format = _G.string.format
local match  = _G.string.match
local insert = _G.table.insert
local remove = _G.table.remove
local type   = _G.type
local huge   = _G.math.huge

------------------------------
-- AddOn Namespace
------------------------------
local LibStub = LibStub

local MODNAME = "Skill Guide"
local addon   = LibStub("AceAddon-3.0"):GetAddon(MODNAME)
local L       = LibStub("AceLocale-3.0"):GetLocale(MODNAME)
local private = select(2, ...)

local function merge(left, right)
   if not left or type(left) ~= "table" then
      if not right or type(right) ~= "table" then
         return nil
      end
      
      return right
   end
   
   if not right or type(right) ~= "table" then
      return left
   end
   
   local result = {}
   while #left > 0 or #right > 0 do
      if #left > 0 and #right > 0 then
         if left[1].cost <= right[1].cost then
            insert(result, remove(left, 1))
         else
            insert(result, remove(right, 1))
         end
      elseif #left > 0 then
         insert(result, remove(left, 1))
      elseif #right > 0 then
         insert(result, remove(right, 1))
      end -- size of array if else chain
   end -- while
end

local function mergeSort(list)
   if not list or type(list) ~= "table" then
      return nil
   end
   
   -- tables to hold the left and right sides
   local left = {}
   local right = {}
   
   local mid = #list / 2
   for k,v in ipairs(list) do
      self:Print("key: " .. k .. " value: " .. v)
      if(k <= mid) then
         
         insert(left, v)
      else
         insert(right, v)
      end
   end
   
  -- left = mergeSort(left)
  -- right = mergeSort(right)
   
  -- return merge(left, right) 
end

function addon:sortOptimalList(list)
   if not list or type(list) ~= "table" then
      self:Print("list is not a table or undefined")
   end
   
   --[[
   every optimal craftable item will be kept in this table
   each item will contain a subtable of reagents and a total cost scalar
   each reagent will have a cost and a number required to craft the item
   ]]--
   local temp = {}
   
   for index, value in ipairs(list) do
      local id = format("%s", value)
      temp[id] = {}
      
      local reagents = { strsplit(";", select(2, DataStore:GetCraftInfo(value))) }
      for k, v in pairs(reagents) do
         local rId, num = match(v, "(%d+)x(%d+)")
         temp[id][rId] = {
            num = num,
            cost = huge,
         }
      end
   end
   
   --[===[@alpha@--
   self:Print("printing the temp table")
   for craftId, v in pairs(temp) do
      self:Print("craftId: " .. craftId)
      for reagentId, value in pairs(v) do
         self:Print("  reagentId: " .. reagentId)
         self:Print("     num: " .. value.num)
         self:Print("     cost: " .. value.cost)
      end
   end
   --@end-alpha@]===]--
   
   -- calculate the cost of each craftable item before doing the mergesort
   
   mergeSort(temp)
end

function addon:BuildOptimalList(profession, spellId)
   if not spellId then
      self:Print("spell id not set, throw error")
   end
   
   -- Get the trade skill table for this spell ID
   local key = format("%s", spellId)
   local skill = self.db.char.tradeskills[key]
   
   -- wipe the old information from the table
   wipe(skill.optimal)
   
   -- insert all the guarenteed skill up recipes into the optimal table
   for i = 1, DataStore:GetNumCraftLines(profession) do
      local isHeader, color, itemId = DataStore:GetCraftLineInfo(profession, i)
      if not isHeader and color == private.constants["optimal"] then
         insert(skill.optimal, itemId)
      end
   end
   
   -- sort the table based on the cost to craft
   addon:sortOptimalList(skill.optimal)
end
